/* $Id: lkst_events.h,v 1.7 2002/04/12 01:00:03 indoh Exp $ */
/* linux/include/linux/lkst_events.h */

/************************************************************
 *
 * COPYRIGHT (C) HITACHI,LTD. 2002 ALL RIGHTS RESERVED.
 * WRITTEN BY HITACHI SYSTEMS DEVELOPMENT LABORATORY,
 *            HITACHI CENTRAL RESEARCH LABORATORY.
 *
 * Created by K.SERIZAWA <serizawa@sdl.hitachi.co.jp>
 *
 ************************************************************/

/*******************************************************
 *
 *  This file may be included twice or more!
 *
 *******************************************************/

	/* The list of mnemonics:
		If LKST_ETYPE_MNEMONICS is defined,
 		the list of LKST_ETYPE_DEF() will become a definition of 
 		lkst_event_type_mnemonics, 
		that is an array of mnemonics of event types.	*/
#if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_DEF_MNEMONICS)&&(!defined LKST_ETYPE_DEF_MNEMONICS_DEFINED))

				/* remember to undef 
				   LKST_ETYPE_DEF{,_ENUM*}		*/
#define	LKST_ETYPE_DEF_TOCLEAN

				/* each element of the array is a pair of 
				   mnemonic and value of event type.	*/
#define LKST_ETYPE_DEF(event_type, prio, hooktype, mnemonic, description)         \
        [(event_type)] = #mnemonic,

				/* The header of the array. */
#define LKST_ETYPE_DEF_ENUM_BEGIN       char *lkst_event_type_mnemonics[] = {

				/* The tail of the enum. It must end with 
				   LKST_ETYPE_MAX.                    	*/
#define LKST_ETYPE_DEF_ENUM_END         [LKST_ETYPE_MAX] = "LKST_ETYPE_MAX" };

				/* Stopper of multi definition. */
#define LKST_ETYPE_DEF_MNEMONICS_DEFINED

#endif /* if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_DEF_MNEMONICS)&&(!defined LKST_ETYPE_DEF_MNEMONICS_DEFINED)) */

/*******************************************************/


	/* The list of descriptions:
		If LKST_ETYPE_DEF_DESCRIPTIONS is defined,
 		the list of LKST_ETYPE_DEF() will become a definition of 
 		lkst_event_type_descriptions, 
		that is an array of descriptions of event types.	*/
#if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_DEF_DESCRIPTIONS)&&(!defined LKST_ETYPE_DEF_DESCRIPTIONS_DEFINED))

				/* remember to undef 
				   LKST_ETYPE_DEF{,_ENUM*}		*/
#define	LKST_ETYPE_DEF_TOCLEAN

				/* each element of the array is a pair of 
				   description and value of event type.	*/
#define LKST_ETYPE_DEF(event_type, prio, hooktype, mnemonic, description)         \
        [(event_type)] = #description,

				/* The header of the array. */
#define LKST_ETYPE_DEF_ENUM_BEGIN       char *lkst_event_type_descriptions[] = {

				/* The tail of the enum. It must end with 
				   LKST_ETYPE_MAX.                    	*/
#define LKST_ETYPE_DEF_ENUM_END         [LKST_ETYPE_MAX] = "LKST_ETYPE_MAX" };

				/* Stopper of multi definition. */
#define LKST_ETYPE_DEF_DESCRIPTIONS_DEFINED

#endif /* if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_DEF_DESCRIPTIONS)&&(!defined LKST_ETYPE_DEF_DESCRIPTIONS_DEFINED)) */

/*******************************************************/


	/* The list of GKHI Declaration of hook header :
		If LKST_ETYPE_DECLARE_HOOKHEADER is defined,
 		the list of LKST_ETYPE_DEF() will become a declaration of 
 		hook header and hook_rec_t data.	*/
#if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_DECLARE_HOOKHEADER)&&(!defined LKST_ETYPE_DECLARE_HOOKHEADER_DEFINED))

				/* remember to undef 
				   LKST_ETYPE_DEF{,_ENUM*}		*/
#define	LKST_ETYPE_DEF_TOCLEAN

				/* each element of the array is a pair of 
				   description and value of event type.	*/
#define LKST_ETYPE_DEF(event_type, prio, hooktype, mnemonic, description) \
        HOOK_DECL_##hooktype(LKST_ETYPE_##mnemonic);

#define HOOK_DECL_NORMAL(etype) LKST_HOOK_DECLARE(etype)
#define HOOK_DECL_INLINE(etype) LKST_HOOK_DECLARE_INLINE(etype)
#define HOOK_DECL_MODULE(etype)
#define HOOK_DECL_UNDEF(etype)

#define LKST_ETYPE_DEF_ENUM_BEGIN
#define LKST_ETYPE_DEF_ENUM_END

				/* Stopper of multi definition. */
#define LKST_ETYPE_DECLARE_HOOKHEADER_DEFINED

#endif /* if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_DECLARE_HOOKHEADER)&&(!defined LKST_ETYPE_DECLARE_HOOKHEADER_DEFINED)) */

/*******************************************************/


	/* The list of GKHI Initialization of hook header :
		If LKST_ETYPE_INIT_HOOK is defined,
 		the list of LKST_ETYPE_DEF() will become a initialization of 
 		hook.	*/
#if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_INIT_HOOK)&&(!defined LKST_ETYPE_INIT_HOOK_DEFINED))

				/* remember to undef 
				   LKST_ETYPE_DEF{,_ENUM*}		*/
#define	LKST_ETYPE_DEF_TOCLEAN

				/* each element of the array is a pair of 
				   description and value of event type.	*/
#define LKST_ETYPE_DEF(event_type, prio, hooktype, mnemonic, description) \
        HOOK_INIT_##hooktype(LKST_ETYPE_##mnemonic)

#define HOOK_INIT_NORMAL(etype) LKST_HOOK_INIT(etype, ret); \
                                retall += ret;
#define HOOK_INIT_INLINE(etype) LKST_HOOK_INIT(etype, ret); \
                                retall += ret;
#define HOOK_INIT_MODULE(etype)
#define HOOK_INIT_UNDEF(etype)


#define LKST_ETYPE_DEF_ENUM_BEGIN
#define LKST_ETYPE_DEF_ENUM_END

				/* Stopper of multi definition. */
#define LKST_ETYPE_INIT_HOOK_DEFINED

#endif /* if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_INIT_HOOK)&&(!defined LKST_ETYPE_INIT_HOOK_DEFINED)) */

/*******************************************************/


	/* The list of GKHI Cleanup of hook header :
		If LKST_ETYPE_TERMINATE_HOOK is defined,
 		the list of LKST_ETYPE_DEF() will become a cleanup of 
 		hook.	*/
#if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_TERMINATE_HOOK)&&(!defined LKST_ETYPE_TERMINATE_HOOK_DEFINED))

				/* remember to undef 
				   LKST_ETYPE_DEF{,_ENUM*}		*/
#define	LKST_ETYPE_DEF_TOCLEAN

				/* each element of the array is a pair of 
				   description and value of event type.	*/
#define LKST_ETYPE_DEF(event_type, prio, hooktype, mnemonic, description) \
        HOOK_TERMINATE_##hooktype(LKST_ETYPE_##mnemonic)

#define HOOK_TERMINATE_NORMAL(etype) LKST_HOOK_TERMINATE(etype, ret); \
                                     retall += ret;
#define HOOK_TERMINATE_INLINE(etype) LKST_HOOK_TERMINATE(etype, ret); \
                                     retall += ret;
#define HOOK_TERMINATE_MODULE(etype)
#define HOOK_TERMINATE_UNDEF(etype)

#define LKST_ETYPE_DEF_ENUM_BEGIN
#define LKST_ETYPE_DEF_ENUM_END

				/* Stopper of multi definition. */
#define LKST_ETYPE_TERMINATE_HOOK_DEFINED

#endif /* if	(!(defined LKST_ETYPE_DEF)&&(defined LKST_ETYPE_TERMINATE_HOOK)&&(!defined LKST_ETYPE_TERMINATE_HOOK_DEFINED)) */

/*******************************************************/


	/* The enumerate of event types:
		By default,
 		the list of LKST_ETYPE_DEF() will become a definition of 
 		lkst_event_type_descriptions, 
		that is an array of descriptions of event types.	*/
#if	(!(defined LKST_ETYPE_DEF))
				/* The list of LKST_ETYPE_DEF()
				   will be a definition of an enum. of
				   event types.				*/

				/* remember to undef 
				   LKST_ETYPE_DEF{,_ENUM*}.		*/
#define	LKST_ETYPE_DEF_TOCLEAN

				/* each element of the enum. is a pair of 
				   a name and value of event type.   	*/
#define	LKST_ETYPE_DEF(event_type, prio, hooktype, mnemonic, description)		\
	LKST_ETYPE_##mnemonic =	(event_type),

				/* The header of the enum. */
#define LKST_ETYPE_DEF_ENUM_BEGIN					\
	typedef enum {

				/* The tail of the enum. It must end with 
				   LKST_ETYPE_MAX.                    	*/
#define LKST_ETYPE_DEF_ENUM_END						\
		LKST_ETYPE_MAX = 0xFFF+0				\
	} lkst_event_type;

				/* remenber not redefine LKST_ETYPE_MAX. */
#define	LKST_ETYPE_MAX_IS_ENUM

#endif /* if	(!(defined LKST_ETYPE_DEF)) */


/*******************************************************/

	/* header */
LKST_ETYPE_DEF_ENUM_BEGIN

	/* Body of definitions of event types */
	/* Syntax:
	   	LKST_ETYPE_DEF(
	   		VALUE_OF_THE_EVENT_TYPE:(0x001..0xffe),
			PRIORITY_OF_THE_EVENT_TYPE:(0x00..0xff),
			TYPE OF GKHI HOOK HEADER,
			MNEMONIC_OF_THE_EVENT_TYPE,
			DESCRIPTION_OF_THE_EVENT_TYPE)			*/

LKST_ETYPE_DEF(0x001, 0x00, NORMAL, PROCESS_CONTEXTSWITCH, PROCESS_CONTEXTSWITCH)
LKST_ETYPE_DEF(0x002, 0x00, INLINE, PROCESS_WAKEUP, PROCESS_WAKEUP)
LKST_ETYPE_DEF(0x003, 0x00, NORMAL, PROCESS_SIGSEND, PROCESS_SIGSEND)
LKST_ETYPE_DEF(0x004, 0x00, NORMAL, PROCESS_LTHREADGEN, PROCESS_LTHREADGEN)

LKST_ETYPE_DEF(0x010, 0x01, NORMAL, INT_HARDWARE_ENTRY, INT_HARDWARE_ENTRY)
LKST_ETYPE_DEF(0x012, 0x01, NORMAL, INT_TASKLETHI_ENTRY, INT_TASKLETHI_ENTRY)
LKST_ETYPE_DEF(0x014, 0x01, NORMAL, INT_TASKLET_ENTRY, INT_TASKLET_ENTRY)
LKST_ETYPE_DEF(0x016, 0x01, NORMAL, INT_BH_ENTRY, INT_BH_ENTRY)

LKST_ETYPE_DEF(0x020, 0x02, INLINE, EXCEPTION_ENTRY, EXCEPTION_ENTRY)
LKST_ETYPE_DEF(0x021, 0x02, INLINE, EXCEPTION_EXIT, EXCEPTION_EXIT)

LKST_ETYPE_DEF(0x030, 0x03, NORMAL, SYSCALL_ENTRY, SYSCALL_ENTRY)
LKST_ETYPE_DEF(0x031, 0x03, NORMAL, SYSCALL_EXIT, SYSCALL_EXIT)

LKST_ETYPE_DEF(0x040, 0x04, NORMAL, FS_DEVRW, FS_DEVRW)
LKST_ETYPE_DEF(0x041, 0x04, NORMAL, FS_DEVEND, FS_DEVEND)
LKST_ETYPE_DEF(0x042, 0x04, NORMAL, FS_BUFBUSY, FS_BUFBUSY)

LKST_ETYPE_DEF(0x050, 0x05, INLINE, MEM_SWAPOUT, MEM_SWAPOUT)
LKST_ETYPE_DEF(0x051, 0x05, NORMAL, MEM_SWAPIN, MEM_SWAPIN)
LKST_ETYPE_DEF(0x052, 0x05, NORMAL, MEM_DO_NOPAGE, MEM_DO_NOPAGE)
LKST_ETYPE_DEF(0x053, 0x05, NORMAL, MEM_DO_WPPAGE, MEM_DO_WPPAGE)
LKST_ETYPE_DEF(0x054, 0x05, NORMAL, MEM_WAIT_PAGE, MEM_WAIT_PAGE)
LKST_ETYPE_DEF(0x055, 0x05, NORMAL, MEM_GET_FREEPAGE, MEM_GET_FREEPAGE)
LKST_ETYPE_DEF(0x056, 0x05, NORMAL, MEM_GET_ZEROPAGE, MEM_GET_ZEROPAGE)
LKST_ETYPE_DEF(0x057, 0x05, NORMAL, MEM_FREEPAGE, MEM_FREEPAGE)
LKST_ETYPE_DEF(0x058, 0x05, INLINE, MEM_VMALLOC, MEM_VMALLOC)
LKST_ETYPE_DEF(0x059, 0x05, NORMAL, MEM_VFREE, MEM_VFREE)
LKST_ETYPE_DEF(0x05a, 0x05, NORMAL, MEM_CACHE_CREATE, MEM_CACHE_CREATE)
LKST_ETYPE_DEF(0x05b, 0x05, NORMAL, MEM_CACHE_ALLOC, MEM_CACHE_ALLOC)
LKST_ETYPE_DEF(0x05c, 0x05, NORMAL, MEM_MALLOC, MEM_MALLOC)
LKST_ETYPE_DEF(0x05d, 0x05, NORMAL, MEM_CACHE_FREE, MEM_CACHE_FREE)
LKST_ETYPE_DEF(0x05e, 0x05, NORMAL, MEM_FREE, MEM_FREE)

LKST_ETYPE_DEF(0x060, 0x06, NORMAL, NET_PKTSEND, NET_PKTSEND)
LKST_ETYPE_DEF(0x061, 0x06, NORMAL, NET_PKTSENDI, NET_PKTSENDI)
LKST_ETYPE_DEF(0x062, 0x06, NORMAL, NET_PKTRECV, NET_PKTRECV)
LKST_ETYPE_DEF(0x063, 0x06, NORMAL, NET_PKTRECVI, NET_PKTRECVI)
LKST_ETYPE_DEF(0x064, 0x06, NORMAL, NET_SOCKETIF, NET_SOCKETIF)

LKST_ETYPE_DEF(0x070, 0x07, NORMAL, SYSV_IPC, SYSV_IPC)

	/* Events concerned with spinlocks should have 
	   event_type as 0x08x. */
LKST_ETYPE_DEF(0x080, 0x08, INLINE, LK_SPINLOCK, LK_SPINLOCK)
LKST_ETYPE_DEF(0x081, 0x08, INLINE, LK_SPINTRYLOCK, LK_SPINTRYLOCK)
LKST_ETYPE_DEF(0x082, 0x08, INLINE, LK_SPINUNLOCK, LK_SPINUNLOCK)
LKST_ETYPE_DEF(0x083, 0x08, INLINE, LK_WRLOCK, LK_WRLOCK)
LKST_ETYPE_DEF(0x084, 0x08, INLINE, LK_WRTRYLOCK, LK_WRTRYLOCK)
LKST_ETYPE_DEF(0x085, 0x08, INLINE, LK_WRUNLOCK, LK_WRUNLOCK)
LKST_ETYPE_DEF(0x086, 0x08, INLINE, LK_RDLOCK, LK_RDLOCK)
LKST_ETYPE_DEF(0x087, 0x08, INLINE, LK_RDUNLOCK, LK_RDUNLOCK)
LKST_ETYPE_DEF(0x088, 0x08, UNDEF,  LK_RESV88, LK_RESV88)
LKST_ETYPE_DEF(0x089, 0x08, UNDEF,  LK_RESV89, LK_RESV89)
LKST_ETYPE_DEF(0x08a, 0x08, UNDEF,  LK_RESV8a, LK_RESV8a)
LKST_ETYPE_DEF(0x08b, 0x08, UNDEF,  LK_RESV8b, LK_RESV8b)
LKST_ETYPE_DEF(0x08c, 0x08, UNDEF,  LK_RESV8c, LK_RESV8c)
LKST_ETYPE_DEF(0x08d, 0x08, UNDEF,  LK_RESV8d, LK_RESV8d)
LKST_ETYPE_DEF(0x08e, 0x08, UNDEF,  LK_RESV8e, LK_RESV8e)
LKST_ETYPE_DEF(0x08f, 0x08, UNDEF,  LK_RESV8f, LK_RESV8f)

LKST_ETYPE_DEF(0x090, 0x09, INLINE, O_PORTIN, O_PORTIN)
LKST_ETYPE_DEF(0x091, 0x09, INLINE, O_PORTOUT, O_PORTOUT)
LKST_ETYPE_DEF(0x092, 0x09, NORMAL, O_PANIC, O_PANIC)
LKST_ETYPE_DEF(0x093, 0x09, NORMAL, O_PRINTK, O_PRINTK)

LKST_ETYPE_DEF(0x0a0, 0x0a, INLINE, TIMER_RUN, TIMER_RUN)
LKST_ETYPE_DEF(0x0a1, 0x0a, NORMAL, TIMER_ADD, TIMER_ADD)
LKST_ETYPE_DEF(0x0a2, 0x0a, NORMAL, TIMER_MOD, TIMER_MOD)
LKST_ETYPE_DEF(0x0a3, 0x0a, NORMAL, TIMER_DEL, TIMER_DEL)
#ifdef CONFIG_SMP
LKST_ETYPE_DEF(0x0a4, 0x0a, NORMAL, TIMER_DEL_SYNC, TIMER_DEL_SYNC)
#endif /* CONFIG_SMP */

LKST_ETYPE_DEF(0xF00, 0xF0, INLINE, LKST_INIT, LKST_INIT)

LKST_ETYPE_DEF(0xF08, 0xF0, INLINE, LKST_MSET_XCHG, LKST_MSET_XCHG)

LKST_ETYPE_DEF(0xF10, 0xF1, INLINE, LKST_BUFF_SHIFT, LKST_BUFF_SHIFT)
LKST_ETYPE_DEF(0xF11, 0xF1, INLINE, LKST_BUFF_OVFLOW, LKST_BUFF_OVFLOW)
LKST_ETYPE_DEF(0xF12, 0xF1, UNDEF,  LKST_BUFF_HIGH, LKST_BUFF_HIGH)

LKST_ETYPE_DEF(0xF18, 0xF1, UNDEF,  LKST_SYNC_TIME, LKST_SYNC_TIME)
LKST_ETYPE_DEF(0xF19, 0xF1, INLINE, LKST_SYNC_UID, LKST_SYNC_UID)
LKST_ETYPE_DEF(0xF1A, 0xF1, INLINE, LKST_SYNC_GID, LKST_SYNC_GID)
LKST_ETYPE_DEF(0xF1B, 0xF1, INLINE, LKST_SYNC_PGID, LKST_SYNC_PGID)

LKST_ETYPE_DEF(0xFFC, 0xFF, UNDEF,  LKST_EXTEND, LKST_EXTEND) 	// TODO: Tantative code
LKST_ETYPE_DEF(0xFFD, 0xFF, UNDEF,  LKST_EXTENDE, LKST_EXTENDE)	// TODO: Tantative code

LKST_ETYPE_DEF(0xFFE, 0xFF, UNDEF,  LKST_OVWRTN_REC, LKST_OVWRTN_REC)

#if	((!defined(LKST_ETYPE_MAX_IS_ENUM))&&(!defined(LKST_ETYPE_MAX)))
				/* By default, LKST_ETYPE_MAX is
				   defined as an element of the enum.
				   Otherwise here LKST_ETYPE_MAX is 
				   defined as a MACRO.			 */
#define LKST_ETYPE_MAX  (0xFFF+0)

#endif /* if	((!defined(LKST_ETYPE_MAX_IS_ENUM))&&(!defined(LKST_ETYPE_MAX))) */

#ifndef	LKST_ETYPE_VOID
#define LKST_ETYPE_VOID  (LKST_ETYPE_MAX+1)
#endif /* ifndef	LKST_ETYPE_VOID */

	/* tail */
LKST_ETYPE_DEF_ENUM_END

				/* By default, undef LKST_ETYPE_DEF* 
				   for the next inclusion of this file.	*/
#ifdef	LKST_ETYPE_DEF_TOCLEAN

#undef	LKST_ETYPE_DEF
#undef	LKST_ETYPE_DEF_ENUM_BEGIN
#undef	LKST_ETYPE_DEF_ENUM_END
#undef	LKST_ETYPE_DEF_TOCLEAN

#endif /* ifdef	LKST_ETYPE_DEF_TOCLEAN */

/* END */
